#!/usr/libexec/platform-python
# -*- coding: UTF-8 -*-

# Copyright (c) 2021 Oracle and/or its affiliates.
#
# The Universal Permissive License (UPL), Version 1.0
#
# Subject to the condition set forth below, permission is hereby granted to any
# person obtaining a copy of this software, associated documentation and/or
# data (collectively the "Software"), free of charge and under any and all
# copyright rights in the Software, and any and all patent rights owned or
# freely licensable by each licensor hereunder covering either (i) the
# unmodified Software as contributed to or provided by such licensor, or (ii)
# the Larger Works (as defined below), to deal in both
#
# (a) the Software, and
#
# (b) any piece of software and/or hardware listed in the lrgrwrks.txt file if
# one is included with the Software (each a “Larger Work” to which the Software
# is contributed by such licensors),
#
# without restriction, including without limitation the rights to copy, create
# derivative works of, display, perform, and distribute the Software and make,
# use, sell, offer for sale, import, export, have made, and have sold the
# Software and the Larger Work(s), and to sublicense the foregoing rights on
# either these or other terms.
#
# This license is subject to the following condition:
#
# The above copyright notice and either this complete permission notice or at a
# minimum a reference to the UPL must be included in all copies or substantial
# portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Modifies Modulemd yaml file from Oracle RPM modular repository to be suitable for use by module-build-service package.
Requires:
  ruamel.yaml module - https://pypi.org/project/ruamel.yaml/
  argparse module - https://pypi.org/project/argparse/
  fileinput module - standard library https://docs.python.org/3/library/fileinput.html

Input:
  Either:
    - STDIN output from `dnf module info <module>:<stream> -v`
  Or:
    - Path of Modulemd yaml file obtained from RPM repository repodata
      This single file can contain multiple yaml documents

Output:
  Modulemd yaml file(s) suitable for use by module-build-service package.
  File(s) name: <name>-<stream>-<version>-<context>.yaml

Usage:
  dnf module info <module>:<stream> -v|./convert_repodata_modulemd.py
  ./convert_repodata_modulemd.py --yaml-file <Modulemd yaml file path>
"""

import sys
# fileinput allows stdin
import fileinput
from argparse import ArgumentParser
# ruamel yaml parses and dumps better
from ruamel.yaml import YAML

def main():
    parser = ArgumentParser(description="Modifies Modulemd yaml from RPM modular repository to be suitable for use by module-build-service package.")
    group = parser.add_mutually_exclusive_group(required=True)
    group.add_argument('--yaml-file', '-f' , action='store', help='Path to an existing yaml file. For example: ./convert_repodata_modulemd.py --yaml-file <Modulemd yaml file path>')
    group.add_argument('--stdin', '-i', default=False, action="store_true", help='Use STDIN to obtain modulemd content. For example: dnf module info <module>:<stream> -v|./convert_repodata_modulemd.py -i')
    arg = parser.parse_args()

    yaml = YAML()
    yaml.default_flow_style = False

    data = ''
    fi_args = {}
    if arg.yaml_file:
        fi_args = arg.yaml_file
    with fileinput.input(fi_args) as f:
        for line in f:
            data += line

    md_list = data.split('---')
    cruft = md_list.pop(0)
    for m in md_list:
        x = yaml.load('---\n'+m)
        if 'xmd' in x['data'].keys():
            del x['data']['xmd']
        if 'artifacts' in x['data'].keys():
            del x['data']['artifacts']
        try:
            name = x["data"]["name"]
            stream = x["data"]["stream"]
            version = x["data"]["version"]
            context = x["data"]["context"]
        except KeyError:
            error_message = "ERROR: name, stream, version and context are expected to be present in the yaml file. Exiting ..."
            sys.exit(error_message)
        with open("%s-%s-%s-%s.yaml" % (name, stream, version, context), 'w') as yaml_file_output:
            yaml.dump(x, yaml_file_output)

if __name__ == '__main__':
    main()
